from http.server import HTTPServer, SimpleHTTPRequestHandler
try:
    from html_json_forms import parse_json_form
except ImportError:
    def parse_json_form(data):
        return parse_values_array(data)


import json
import random
import string
import cgi


batch_number = 0


class RequestHandler(SimpleHTTPRequestHandler):
    def echo(self, status=200, id=None):
        ctype, pdict = cgi.parse_header(self.headers.get('content-type'))
        pdict = {key: val.encode('utf-8') for key, val in pdict.items()}
        form = cgi.parse_multipart(self.rfile, pdict)
        data = parse_json_form({
            key: val[0].decode('utf-8')
            for key, val in form.items()
        })
        for value in data.values():
            if isinstance(value, list):
                for i, val in enumerate(value):
                    val['@index'] = i
        if id and 'id' not in data:
            data['id'] = id
        self.send_response(status)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(json.dumps(data).encode('utf-8'))

    def do_POST(self):
        if '/error' in self.path or 'status/400' in self.path:
            self.status_400()
        elif 'status/500' in self.path:
            self.status_500()
        elif 'reset-batch-number' in self.path:
            self.reset_batch_number()
        elif 'batch' in self.path:
            self.process_batch()
        else:
            self.status_200()

    def generate_id(self):
        return ''.join([
            random.choice(string.ascii_lowercase)
            for o in range(3)
        ])

    def status_200(self):
        self.echo(id=self.generate_id())

    def status_400(self):
        self.send_response(400)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(json.dumps({
            "label": ["Not a valid label"]
        }).encode('utf-8'))

    def status_500(self):
        self.send_response(500)
        self.send_header("Content-Type", "text/html")
        self.end_headers()
        self.wfile.write(b"SERVER ERROR")

    def reset_batch_number(self):
        global batch_number
        batch_number = 0
        self.send_response(200)
        self.end_headers()
        self.wfile.write(b"{}")

    def process_batch(self):
        global batch_number
        batch_number += 1
        size = int(self.headers['Content-Length'])
        requests = json.loads(self.rfile.read(size).decode('utf-8'))

        def process_request(request):
            data = json.loads(request['body'])
            data['id'] = self.generate_id()
            data['batch'] = batch_number
            return {
                'status_code': 200,
                'headers': {},
                'body': json.dumps(data)
            }

        responses = [
            process_request(request) for request in requests
        ]

        self.send_response(200)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(json.dumps(responses).encode('utf-8'))

    def do_PUT(self):
        pk = self.path.split('/')[-1].split('.')[0]
        self.echo(id=pk)

    def do_OPTIONS(self):
        self.send_response(200)
        self.send_header("Content-Type", "text/plain")
        self.end_headers()
        self.wfile.write(b"OK")

    def end_headers(self):
        self.send_header("Access-Control-Allow-Origin", "*")
        super().end_headers()


def run():
    address = ("", 8080)
    httpd = HTTPServer(address, RequestHandler)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        httpd.server_close()


def parse_values_array(data):
    for key, val in list(data.items()):
        if key.startswith('values['):
            i = int(key[7])
            if 'values' not in data:
                data['values'] = []
            while len(data['values']) < i + 1:
                data['values'].append({})
            data['values'][i][key[9:].strip('[]')] = data[key]
            data.pop(key)
    return data


if __name__ == "__main__":
    run()



from wq.core import wq
import click
import os
import json
from glob import glob


@wq.command()
@click.argument('version')
@click.option(
    '--template',
    type=click.Path(),
    help="Override service worker template"
)
@click.option(
    '--output',
    type=click.Path(),
    default="./htdocs/service-worker.js",
    help="Destination file (default: ./htdocs/service-worker.js)"
)
@click.option(
    '--cache',
    type=click.Path(),
    multiple=True,
    help="File(s) or directories to cache"
)
@click.option(
    '--scope',
    type=str,
    default="/",
    help="Path prefix for service worker and cached files"
)
@click.option(
    '--timeout',
    type=int,
    default=400,
    help="Timeout to use before falling back to cache."
)
def serviceworker(version, template, output, cache, scope, timeout):
    """
    Generate a service-worker.js.  The default service worker will function as
    follows:

    \b
    1. While installing, precache all of the listed paths.
    2. While running, attempt to fetch (and re-cache) the latest version
       of each file, but fall back to the cached version after the specified
       timeout.

    This ensures the app can work offline as needed, without requiring a double
    refresh whenever a new version is available.  Specify the --template option
    to override the default.

    The list of files to cache should be relative to the directory containing
    service-worker.js.  Wildcard paths will be resolved to existing filenames,
    so this command should generally be run *after* ./manage.py collectstatic.

    Note: If you are using wq with npm, a service worker will be generated by
    create-react-app, so you do not need to use this command.
    """

    if template:
        with open(template) as f:
            template = f.read()
    else:
        template = SW_TMPL

    if not scope.endswith('/'):
        scope += '/'

    basedir = os.path.dirname(output)
    paths = []

    for path in cache:
        if path.startswith('/'):
            path = path[1:]
        if '*' not in path:
            paths.append(scope + path)
            continue
        for filename in glob(os.path.join(basedir, path)):
            paths.append(filename.replace(basedir + '/', scope))

    cache = ",".join(json.dumps(path) for path in paths)

    with open(output, 'w') as f:
        f.write(
            template.replace("{{VERSION}}", version)
            .replace("{{CACHE}}", cache)
            .replace("{{TIMEOUT}}", str(timeout))
        )


SW_TMPL = """const CACHE_NAME = 'wq-cache-v1';

// App Version {{VERSION}}

const cacheUrls = [{{CACHE}}];

self.addEventListener('install', event => {
    event.waitUntil(
        caches.open(CACHE_NAME).then(cache => cache.addAll(cacheUrls))
    );
});

self.addEventListener('fetch', event => {
    if (!cacheUrls.includes(new URL(event.request.url).pathname)) {
        return;
    }
    event.respondWith(
        new Promise((resolve, reject) => {
            const timeout = setTimeout(reject, {{TIMEOUT}});
            fetch(event.request).then(response => {
                clearTimeout(timeout);
                const cacheResponse = response.clone();
                caches
                    .open(CACHE_NAME)
                    .then(cache => cache.put(event.request, cacheResponse));
                resolve(response);
            }, reject);
        }).catch(() => {
            return caches
                .open(CACHE_NAME)
                .then(cache => cache.match(event.request))
                .then(response => response || Promise.reject('no-match'));
        })
    );
});
"""
